class JsonUtil(object):
    pass
    # def getAppJson(self, name):
    #     """ generated source for method getAppJson """
    #     result1 = ""
    #     try:
    #         result1 = fileinput.input()
    #         # result1 = IOUtils(
    #         #     FileInputStream(File(System.getProperty("user.dir") + "/src/test/resources/" + name + ".json")))
    #     except Exception as e:
    #         print(e)
    #     result = JSONObject()
    #     return result



from time import sleep

from test.DriverFactory import DriverFactory


class TestPageTitle(DriverFactory):
    def test_page_title_1(self):
        sleep(3)
        self.driver.find_element_by_id("classic_bottom_navigation_icon").click()
        assert self.driver.find_element_by_id("headerTxt").text == "Search Flights"

    def test_page_title_2(self):
        sleep(3)
        self.driver.find_element_by_id("classic_bottom_navigation_icon").click()
        assert self.driver.find_element_by_id("headerTxt").text == "Search Flights"

    def test_page_title_3(self):
        sleep(3)
        self.driver.find_element_by_id("classic_bottom_navigation_icon").click()
        assert self.driver.find_element_by_id("headerTxt").text == "Search Flights"



from clients.AkiraClient import AkiraClient
from entities.SessionInfo import SessionInfo
from entities.SessionReservationDetails import SessionReservationDetails
from entities.SessionsByPlatform import SessionsByPlatform
from utils.BuildNoGenerator import BuildNoGenerator

akiraClient = AkiraClient()


class OptimusCloudManager(object):

    def __int__(self):
        pass

    @classmethod
    def getBuildNo(cls, buildNo):
        if buildNo is None:
            return BuildNoGenerator().getBuildNo()
        else:
            return buildNo

    def reserveSession(
            self,
            buildNo,
            numOfSessionsToBeReserved,
            platformsToBeReserved):
        buildNo = self.getBuildNo(buildNo)
        sessionReservationDetails = SessionReservationDetails()
        sessionReservationDetails.setBuildNo = buildNo
        sessionsByPlatform = SessionsByPlatform()
        sessionsByPlatform.platformName = platformsToBeReserved
        sessionsByPlatform.sessionsRequired = numOfSessionsToBeReserved
        sessionReservationDetails.sessionDetails = sessionsByPlatform
        sessionsReserved = akiraClient.reserveSessions(
            sessionReservationDetails)
        sessionInfo = SessionInfo()
        sessionInfo.buildNo = buildNo
        sessionInfo.sessionsReserved = sessionsReserved.totalSessions
        return sessionInfo

    def reserveAndroidSession(
            self,
            buildNo=None,
            numOfAndroidSessionsToBeReserved=None):
        return self.reserveSession(
            buildNo, numOfAndroidSessionsToBeReserved, "Android")

    def reserveIosSession(
            self,
            buildNo=None,
            numOfAndroidSessionsToBeReserved=None):
        return self.reserveSession(
            buildNo, numOfAndroidSessionsToBeReserved, "IOS")

    def getSessionsByPlatform(self, platformName, numOfSessionsToBeReserved):
        sessionsByPlatform = SessionsByPlatform()
        sessionsByPlatform.platformName = platformName
        sessionsByPlatform.sessionsRequired = numOfSessionsToBeReserved
        return sessionsByPlatform

    def reserveAndroidAndIosSessions(
        self,
        noOfAndroidSessions,
        noOfIOSSessions,
        buildNo=None,
    ):
        buildNo = self.getBuildNo(buildNo)
        sessionReservationDetails = SessionReservationDetails()
        sessionReservationDetails.setBuildNo = buildNo
        androidSession = self.getSessionsByPlatform(
            "Android", noOfAndroidSessions)
        iOSSession = self.getSessionsByPlatform("IOS", noOfIOSSessions)
        SessionReservationDetails.sessionDetails = [androidSession, iOSSession]
        return akiraClient.reserveSessions(sessionReservationDetails)

    @classmethod
    def unReserveSession(cls, buildNo):
        akiraClient.unReserveSession(buildNo)

    @classmethod
    def engageSession(cls, sessionUrl):
        return akiraClient.engageSession(sessionUrl)

    @classmethod
    def releaseSession(cls, mobileDriverDetails):
        return akiraClient.releaseSession(mobileDriverDetails.sessionUrl)

    @classmethod
    def terminateSession(cls, sessionUrl):
        akiraClient.terminateSession(sessionUrl)

    @classmethod
    def getSessionState(cls, sessionUrl):
        return akiraClient.getCurrentSessionState(sessionUrl)



import requests


def isSessionUp(sessionUrl):
    try:
        requests.get(sessionUrl)
        return True
    except Exception as error:
        print(error)
        return False


class SessionManager:
    pass


if __name__ == '__main__':
    print(isSessionUp('http://192.168.0.188:52419/wd/hub'))



import requests

from configuration import OptimusCloudConfiguration
from entities.MandatoryCaps import MandatoryCaps
from entities.SessionReservationDetails import SessionReservationDetails


def manageSession(sessionUrl, endpoint):
    return requests.put(endpoint,
                        params={"sessionUrl": sessionUrl})


class AkiraClient:
    HOST = OptimusCloudConfiguration.OptimusCloudConfiguration.AKIRA_HOST
    RESERVE_SESSION = HOST + "/reserveSession"
    UN_RESERVE_SESSION = HOST + "/unReserveSession"
    SESSION_STATE = HOST + "/sessionState"
    SESSION_CURRENT_STATE = "/currentState"
    ENGAGE_SESSION = SESSION_STATE + "/engage"
    RELEASE_SESSION = SESSION_STATE + "/release"
    TERMINATE_SESSION = SESSION_STATE + "/terminate"
    SESSION_CAPABILITIES = HOST + "/sessionCapabilities"
    FIND_MATCHING_SESSION_HOST = HOST + "/findMatchingSession"
    WFSMBOX = HOST + "/wfsmbox"

    @classmethod
    def getSessionDetails(cls, mandatoryCaps: MandatoryCaps):
        matchingSessionDetails = requests.get(
            AkiraClient.FIND_MATCHING_SESSION_HOST,
            params={"platformName": mandatoryCaps.platformName,
                    "deviceName": mandatoryCaps.getDeviceName(),
                    "platformVersion": mandatoryCaps.getPlatformVersion(),
                    "udid": mandatoryCaps.getUdid(),
                    "buildNo": mandatoryCaps.buildNo,
                    "deviceType": mandatoryCaps.getDeviceType()
                    },
        )
        print("FindMatchingSession" + AkiraClient.FIND_MATCHING_SESSION_HOST)
        return matchingSessionDetails.text

    @classmethod
    def getCurrentSessionState(cls, sessionUrl):
        sessionStateResponse = requests \
            .put(AkiraClient.SESSION_CURRENT_STATE,
                 params={"sessionUrl": sessionUrl})
        return sessionStateResponse.content

    @classmethod
    def engageSession(cls, sessionUrl):
        manageSession(sessionUrl, AkiraClient.ENGAGE_SESSION)

    @classmethod
    def releaseSession(cls, sessionUrl):
        manageSession(sessionUrl, AkiraClient.RELEASE_SESSION)

    @classmethod
    def terminateSession(cls, sessionUrl):
        manageSession(sessionUrl, AkiraClient.TERMINATE_SESSION)

    @classmethod
    def getSessionCapabilities(cls, sessionUrl):
        sessionCapabilitiesResponse = requests.get(
            AkiraClient.SESSION_CAPABILITIES,
            params={"sessionUrl": sessionUrl})
        return sessionCapabilitiesResponse.__dict__

    @classmethod
    def reserveSessions(cls, sessionReservationDetails):
        requests.post(
            AkiraClient.SESSION_CAPABILITIES,
            data={sessionReservationDetails})
        return SessionReservationDetails.__dict__

    @classmethod
    def unReserveSession(cls, buildNo):
        requests.post(
            AkiraClient.UN_RESERVE_SESSION,
            params={"buildNo", buildNo}
        )



from entities.SessionState import SessionState


class SessionDetails(object):
    desiredCapabilities = {}
    sessionUrl = str()
    sessionState = SessionState()
    isReserved = str()

    def __str__(self):
        return "{" + "desiredCapabilities={}".format(self.desiredCapabilities) \
               + ", sessionUrl={}".format(self.sessionUrl) \
               + ", sessionState={}".format(self.sessionState) \
               + ", isReserved={}".format(self.isReserved) + "}"



import json
from venv import logger

from atomos import atomic
from retrying import retry
from selenium.common.exceptions import SessionNotCreatedException

from clients.AkiraClient import AkiraClient
from driver.DriverCreatorFactory import DriverCreatorFactory
from driver.SessionManager import isSessionUp
from entities.MobileDriverDetails import MobileDriverDetails
from entities.SessionDetails import SessionDetails
from exceptions.EmptySessionUrlException import EmptySessionUrlException
from utils import CapToManCapConverter


class OptimusCloudDriver:

    def __init__(self):
        self.akiraClient = AkiraClient()

    def createDriver(self, buildNo=None, desiredCapabilities=None):
        # if testFeed is None:
        print("Creating driver with capabilities = {0}".format(
            desiredCapabilities))
        mandatoryCaps = CapToManCapConverter.convert(
            buildNo, desiredCapabilities=desiredCapabilities)
        # if desiredCapabilities is None:
        #     mandatoryCaps = TestFeedToDesiredCapConverter(testFeed).convert()
        sessionDetails = self.getSessionDetailsFromCloud(mandatoryCaps)
        print("Session details = {0}".format(sessionDetails))
        return self.initDriver(desiredCapabilities, sessionDetails)

    # def getSessionDetailsIfSessionIsUnAssigned(self, buildNo, sessionDetails):
    #     if sessionDetails.sessionUrl is None:
    #         sessionUrl = self.waitTillSessionIsAvailable(buildNo)
    #     else:
    #         return sessionDetails
    #     sessionCapabilities = self.akiraClient.getSessionCapabilities(sessionUrl)
    #     sessionDetailsUpdated = SessionDetails()
    #     sessionDetails.sessionUrl = sessionUrl
    #     sessionDetails.desiredCapabilities = sessionCapabilities
    #     sessionDetails.isReserved = True
    #     sessionDetails.sessionState = SessionState.AVAILABLE
    #     return sessionDetailsUpdated

    def initDriver(self, desiredCapabilities, sessionDetails):
        try:
            return self.getDriverDetails(desiredCapabilities, sessionDetails)
        except SessionNotCreatedException as error:
            self.akiraClient.terminateSession(sessionDetails.sessionUrl)
            logger.error(error)
            raise Exception('Cannot initialize driver')

    # def waitTillSessionIsAvailable(self, buildNo):
    #     sessionUrl = ""
    #     sessionQueueReader = SessionQueueReader()
    #     isMessageRead = False
    #     while not isMessageRead:
    #         poll(
    #             lambda: queueHasAMessage(buildNo),
    #             timeout=120,
    #             step=5
    #         )
    #         sessionUrl = consumeMessage(buildNo)
    #         if sessionUrl is None or not sessionUrl:
    #             isMessageRead = True
    #             self.akiraClient.engageSession(sessionUrl)
    #     return sessionUrl

    @staticmethod
    def getMobileDriverDetails(
            sessionDetails: SessionDetails(),
            desiredCapabilities,
            mobileDriver):
        mobileDriverDetails = MobileDriverDetails()
        mobileDriverDetails.sessionUrl = (sessionDetails.get("sessionUrl"))
        mobileDriverDetails.mobileDriver = mobileDriver
        mobileDriverDetails.udid = (
            sessionDetails.get("desiredCapabilities").get("udid"))
        mobileDriverDetails.desiredCapabilities = desiredCapabilities
        return mobileDriverDetails

    @classmethod
    def getDesiredCapabilities(cls, desiredCapabilities: dict):
        desiredCapabilities1 = {}
        for key, value in desiredCapabilities.items():
            desiredCapabilities1[key] = value
        return desiredCapabilities1

    @retry(stop_max_attempt_number=100, wait_fixed=4000)
    def getSessionDetailsFromCloud(self, mandatoryCaps):
        sessionDetails = atomic.AtomicReference(SessionDetails)
        try:
            sessionDetails.set(
                self.akiraClient.getSessionDetails(mandatoryCaps))
            print("Mandatory caps ={}".format(mandatoryCaps))
            print("Session details ={}".format(sessionDetails.get()))
            sessionDetail = json.loads(sessionDetails.get())
            sessionUrl = sessionDetail.get("sessionUrl")
            if sessionUrl is None:
                raise EmptySessionUrlException()
            sessionUp = isSessionUp(sessionUrl)
            if not sessionUp:
                self.akiraClient.terminateSession(sessionUrl)
                raise EmptySessionUrlException()
            return sessionDetail
        except Exception as error:
            print(error)
            raise print("Retrying session")

    @retry(stop_max_attempt_number=100)
    def getDriverDetails(self, desiredCapabilities, sessionDetails):
        for key, value in sessionDetails.get("desiredCapabilities").items():
            desiredCapabilities[key] = value
        mobileDriver = DriverCreatorFactory().getInstance(
            sessionDetails.get("sessionUrl"), desiredCapabilities)
        return self.getMobileDriverDetails(
            sessionDetails, desiredCapabilities, mobileDriver)



from json.decoder import JSONObject

from utils.JsonUtil import JsonUtil


class TestFeedToDesiredCapConverter:
    appJson = JSONObject

    def __init__(self, testFeed):
        jsonUtil = JsonUtil
        self.appJson = jsonUtil.getAppJson(testFeed)

    # def convert(self):
    #     desiredCapabilitiesList = []
    #     testFeedArray = self.appJson.get("testFeed")
    #     for i in range(0, len(testFeedArray)):
    #         testFeedJSON = testFeedArray.get(i)
    #         print("Updated testFeed " + testFeedJSON)
    #         desiredCapabilities = CapabilitiesBuilder(testFeedJSON).buildCapabilities()
    #         desiredCapabilitiesList.append(desiredCapabilities)
    #     return desiredCapabilitiesList



class SessionsByPlatform(object):
    platformName = str
    sessionsRequired = int
    sessionsAllocated = int

    def __str__(self):
        return "{platformName={}".format(self.platformName) + \
               ", sessionsRequired={}".format(self.sessionsRequired) + \
               ", sessionsAllocated={}".format(self.sessionsAllocated) + '}'



from appium import webdriver


class MobileDriverDetails(object):
    mobileDriver = webdriver.Remote
    sessionUrl = str

    # For Optimus Usage
    udid = str

    # For dashboard usage
    desiredCapabilities = {}



import pika
from pika import channel
from pika.channel import Channel


def getChannel():
    return channel.Channel


def test():
    connection = pika.BlockingConnection(pika.ConnectionParameters(host='localhost'))
    channel = connection.channel()
    channel.queue_declare(queue='hello')

    def callback(ch, method, properties, body):
        print(" [x] Received %r" % body)

    channel.basic_consume(queue='hello', on_message_callback=callback, auto_ack=False, exclusive=False,
                          consumer_tag=None, arguments=None, )

    print(' [*] Waiting for messages. To exit press CTRL+C')
    # channel.start_consuming()


class SessionConnection(object):
    pass


def getInstance(sessionConnection=None):
    if sessionConnection is None:
        sessionConnection = SessionConnection7()
    return sessionConnection


def getConnection():
    connection = pika.BlockingConnection(
        pika.ConnectionParameters(host="localhost")
    )
    return connection


class SessionConnection7(object):
    sessionConnection = SessionConnection()
    channel = Channel

    def getChannel(self):
        return self.channel

    def __init__(self):
        self.sessionConnection = SessionConnection7
        connection = getConnection()
        channel = connection.channel()
        # can specify various properties that control the durability of the queue and its contents, and the level of
        # sharing for the queue.
        channel.queue_declare(queue='hello')
        channel.basic_publish(exchange='', routing_key='hello', body='Hello World!')
        print("[x] Sent 'Hello World!'")
        connection.close()


if __name__ == '__main__':
    SessionConnection7()
    test()
    print(getChannel())


class UserInfo(object):
    username = str
    password = str



class OptimusCloudConfiguration(object):
    AKIRA_HOST = "http://localhost:6889"
    MDB_HOST = "http://localhost:9909"
    MATCH_MAKER_HOST = "http://localhost:6898"
    RABBIT_MQ_HOST = "http://localhost:15672"



import pytest

from remote.OptimusCloudDriver import OptimusCloudDriver
from remote.OptimusCloudManager import OptimusCloudManager


class DriverFactory(object):
    @pytest.fixture(scope='function', autouse=True)
    def setUp(self) -> None:
        desired_caps = {
            'platformName': 'Android',
            'appPackage': 'com.cleartrip.android',
            'appActivity': 'com.cleartrip.android.activity.common.SplashActivity'
        }
        self.mobileDriverDetails = OptimusCloudDriver().createDriver(desiredCapabilities=desired_caps)
        self.driver = self.mobileDriverDetails.mobileDriver

    @pytest.fixture(scope='function', autouse=True)
    def tearDown(self) -> None:
        OptimusCloudManager().releaseSession(self.mobileDriverDetails)



from json.decoder import JSONArray, JSONObject


def isBrowserAppProvided(testFeedJSON):
    try:
        testFeedJSON.getString("app")
    except Exception as error:
        print(error)
        return False
    return True


def setDesiredCapabilities(platformSpecificCapabilities, desiredCapabilities):
    keys = platformSpecificCapabilities.keys()
    while keys.hasNext():
        key = keys.next()
        if key.lower() == "app".lower():
            continue
        value = platformSpecificCapabilities.get(key)
        if isinstance(value, bool):
            desiredCapabilities.setCapability(key, platformSpecificCapabilities.getBoolean(key))
        elif isinstance(value, str):
            desiredCapabilities.setCapability(key, platformSpecificCapabilities.get(key))
        elif isinstance(value, int):
            desiredCapabilities.setCapability(key, platformSpecificCapabilities.getInt(key))
        elif isinstance(value, JSONArray):
            desiredCapabilities.setCapability(key, platformSpecificCapabilities.getJSONArray(key))


def isAndroid(testFeedJSON):
    platformName = testFeedJSON.get("platformName")
    return platformName.lower() == "Android".lower()


class CapabilitiesBuilder(object):
    capabilities = dict
    testFeedJSON = JSONObject

    def __init__(self, testFeedJSON):
        self.testFeedJSON = testFeedJSON
        appiumServerCapabilities = (testFeedJSON.get("optimusDesiredCapabilities")).get("appiumServerCapabilities")
        if not self.isNativeApp():
            self.buildWebAppCapabilities(appiumServerCapabilities)
            return
        appPath = self.getAppPath(str(testFeedJSON.get("appDir")), str(appiumServerCapabilities.get("app")))
        self.capabilities["app"] = appPath
        self.initializeCapabilities()

    def buildWebAppCapabilities(self, appiumServerCapabilities):
        appPath = self.getAppPath(str(appiumServerCapabilities["appDir"]), str(appiumServerCapabilities.get("app")))
        if isBrowserAppProvided(appiumServerCapabilities):
            self.capabilities["app"] = appPath
        self.initializeCapabilities()

    def isNativeApp(self):
        nativeApp = self.testFeedJSON.getBoolean("nativeApp")
        return nativeApp

    def initializeCapabilities(self):
        appiumServerCapabilities = (self.testFeedJSON.get("optimusDesiredCapabilities")).get("appiumServerCapabilities")
        platformSpecificCapabilities = None
        if appiumServerCapabilities.get("platformName").__str__().lower() == "Android".lower():
            platformSpecificCapabilities = (self.testFeedJSON.get("optimusDesiredCapabilities")).get(
                "androidOnlyCapabilities")
        elif appiumServerCapabilities.get("platformName").__str__().lower() == "iOS".lower():
            platformSpecificCapabilities = (self.testFeedJSON.get("optimusDesiredCapabilities")).get(
                "iOSOnlyCapabilities")
        setDesiredCapabilities(appiumServerCapabilities, self.capabilities)
        setDesiredCapabilities(platformSpecificCapabilities, self.capabilities)

    def buildCapabilities(self):
        return self.capabilities



from entities.MandatoryCaps import MandatoryCaps


def convert(buildNo, desiredCapabilities):
    mandatoryCaps = MandatoryCaps()
    mandatoryCaps.buildNo = buildNo
    mandatoryCaps.platformName = desiredCapabilities.get("platformName")
    mandatoryCaps.platformVersion = desiredCapabilities.get("platformVersion")
    mandatoryCaps.deviceType = desiredCapabilities.get("deviceType")
    mandatoryCaps.udId = desiredCapabilities.get("udid")
    mandatoryCaps.deviceName = desiredCapabilities.get("deviceName")
    return mandatoryCaps


class CapToManCapConverter(object):
    pass



class SessionInfo(object):
    buildNo = int
    sessionsReserved = str()

    def __str__(self):
        return "{buildNo={}".format(self.buildNo) \
               + ", sessionsReserved={}".format(self.sessionsReserved) + "}"



from threading import Lock

from messaging.SessionConnection import getInstance, getConnection, getChannel

lock = Lock()


def consumeMessage(routingKey):
    lock.acquire()
    try:
        channel = getChannel()

        getResponse = channel.basic_get(routingKey, False)
        return getResponse.__str__
    except Exception as error:
        return error


def queueHasAMessage(queue_name):
    instance = getInstance().getChannel()
    connection = getConnection().channel()
    queue = connection.queue_declare(queue=queue_name).method.message_count
    return queue > 0


class SessionQueueReader:
    pass


if __name__ == '__main__':
    consumeMessage('hello')
    print(queueHasAMessage('hello'))



from appium import webdriver


class AndroidDriverCreator(object):

    @classmethod
    def getDriver(cls, sessionUrl, desiredCapabilities):
        print("sessionUrl - " + sessionUrl)
        return webdriver.Remote(sessionUrl, desiredCapabilities)



import setuptools
import unittest

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="Optimus-Python-Client",
    version="0.3.3",
    author="Nishant Sharma",
    author_email="thenishant3@gmail.com",
    description="python-client for optimusCloud",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/testvagrant/optimus-pleiades/optimus-cloud/clients/python-client",
    packages=setuptools.find_packages(),
    include_package_data=True,
    install_requires=["polling", "retrying", "requests", "pika", "atomos", "selenium", "Appium-Python-Client"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6'
)



class MandatoryCaps(object):
    def __init__(self):
        self.platformName = str()
        self.deviceName = str
        self.deviceType = str()
        self.platformVersion = str()
        self.udId = str
        self.buildNo = str()

    def getDeviceName(self):
        if self.deviceName == 'None':
            return ''

    def getDeviceType(self):
        if self.deviceType == 'None':
            return ''

    def getPlatformVersion(self):
        if self.platformVersion == 'None':
            return ''

    def getUdid(self):
        if self.udId == 'None':
            return ''

    def __str__(self):
        return " platformName={}".format(self.platformName) \
               + ",deviceName={}".format(MandatoryCaps.getDeviceName(self)) \
               + " ,deviceType={}".format(MandatoryCaps.getDeviceType(self)) \
               + " ,platformVersion={}".format(MandatoryCaps.getPlatformVersion(self)) \
               + " ,udid={}".format(MandatoryCaps.getUdid(self)) \
               + " ,buildNo={}".format(self.buildNo)



from driver.AndroidDriverFactory import AndroidDriverCreator
from driver.IosDriverFactory import IosDriverCreator


class DriverCreatorFactory(object):

    @classmethod
    def getInstance(cls, sessionUrl, desiredCapabilities):
        if desiredCapabilities.get("platformName").__str__().lower() == "android":
            return AndroidDriverCreator().getDriver(sessionUrl, desiredCapabilities)
        elif desiredCapabilities.get("platformName").__str__().lower() == "ios":
            return IosDriverCreator().getDriver(sessionUrl, desiredCapabilities)
        else:
            raise Exception("Cannot find platformName capability to create a new driver")



from appium import webdriver


class IosDriverCreator(object):

    @classmethod
    def getDriver(cls, sessionUrl, desiredCapabilities):
        return webdriver.Remote(sessionUrl, desiredCapabilities)
